from django.contrib.auth.models import Group
from django.test import TestCase

from core.models import Submission, SubmissionType, Feedback
from util.importer import GradyUserFactory


def ensure_groups():
    Group.objects.get_or_create(name='Tutors')
    Group.objects.get_or_create(name='Students')
    Group.objects.get_or_create(name='Reviewers')


class FeedbackTestCase(TestCase):

    factory = GradyUserFactory()

    @classmethod
    def setUpTestData(cls):
        ensure_groups()

    def setUp(self):
        self.tutor   = self.factory.make_tutor()
        self.student = self.factory.make_student()

        submission_type = SubmissionType.objects.create(
            name='Cooking some crystal with Jesse')
        Submission.objects.create(student=self.student, type=submission_type)
        Submission.assign_tutor(self.tutor)

    def test_can_assign_tutor(self):
        self.assertEqual(self.tutor.feedback_list.count(), 1)

    def test_feedback_origin_is_manual(self):
        feedback = self.tutor.feedback_list.all()[0]
        self.assertEqual(feedback.origin, Feedback.MANUAL)

    def test_feedback_status_is_editable(self):
        feedback = self.tutor.feedback_list.all()[0]
        self.assertEqual(feedback.status, Feedback.EDITABLE)


class FactoryTestCase(TestCase):

    factory = GradyUserFactory()

    @classmethod
    def setUpTestData(cls):
        ensure_groups()

    def test_make_student(self):

        student = self.factory.make_student()

        self.assertEqual(student.user.groups.filter(name='Students').count(), 1)
        self.assertEqual(student.exam, None)
        self.assertEqual(len(str(student.matrikel_no)), 8)

    def test_can_create_reviewer(self):
        self.assertTrue(Group.objects.get(name='Reviewers')
                        in self.factory.make_reviewer().groups.all())

    def test_can_create_tutor(self):
        self.assertTrue(Group.objects.get(name='Tutors')
                        in self.factory.make_tutor().groups.all())

    def test_can_create_student(self):
        self.assertTrue(Group.objects.get(name='Students')
                        in self.factory.make_student().user.groups.all())
