from django.contrib import messages
from django.contrib.auth import authenticate, login
from django.contrib.auth.views import LoginView, LogoutView
from django.http import HttpResponseRedirect
from django.urls import reverse, reverse_lazy

from core.custom_annotations import in_groups


def is_student(user):
    return in_groups(user, ('Students',))

def is_deactivated_student(user) -> bool:
    """Checks if the user is linked to a student and if the field user
    has_logged_in is True or not. Obviously a superuser should not be a student.

    Args:
        user (User object): The original user of some request.

    Returns:
        bool: True if user is a student and has already logged in
    """
    if user.is_superuser:
        return False

    if in_groups(user, ('Students',)):
        return user.student.has_logged_in

class Login(LoginView):

    success_url = reverse_lazy('start')
    template_name = 'core/index.html'

    def get(self, request):
        return HttpResponseRedirect(reverse('index'))

    def form_valid(self, form):
        username = form.cleaned_data['username']
        password = form.cleaned_data['password']

        user = authenticate(username=username, password=password)

        if user is not None and user.is_active and not is_deactivated_student(user):
            login(self.request, user)

            # disable the user if s/he is a student
            if is_student(user) and not user.is_superuser:
                user.student.disable()

            print(self.get_success_url())
            return HttpResponseRedirect(self.get_success_url())

        # Handle all the errors separately
        if is_deactivated_student(user) or not user.is_active:
            messages.warning(self.request, "Your Grady account has been deactivated.")
        elif not user:
            # Bad login details were provided. So we can't log the user in.
            print("Invalid login details: {0}, {1}".format(username, password))
            messages.error(self.request, "Invalid login details supplied.")

        return super().form_invalid(form)

class Logout(LogoutView):
    next_page = reverse_lazy('index')
